VERSION 5.00
Begin VB.UserControl FastGraph 
   BackColor       =   &H00808080&
   ClientHeight    =   4170
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   5355
   ScaleHeight     =   4170
   ScaleWidth      =   5355
End
Attribute VB_Name = "FastGraph"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'////////////////////////////////////////////////
'// for fast graph drawing task
'// based on Graph.ctl design
'// limitation: no line thickness
'// using MBITMAP soa's made line algol
'// 18 March 2011
'//
'////////////////////////////////////////////////
'// dependency:
'// MultimediaBitmap.bas
'// MultimediaPainter.bas
'//
'////////////////////////////////////////////////
'// properties
'//
'// Handle
'// Dimension
'//
'// AutoRedraw
'// BackColor
'//
'// GridColor
'// OriginColor
'// GraphColor
'// GridStyle
'// OriginStyle
'// GraphStyle
'//
'// XMin
'// XMax
'// YMin
'// YMax
'//
'// XGrid
'// YGrid
'// XDiv
'// YDiv
'//
'////////////////////////////////////////////////
'// events
'//
'// Paint
'// Resize
'// MouseDown
'// MouseMove
'// MouseUp
'//
'////////////////////////////////////////////////
'// member functions
'//
'// Limit
'// Grid
'//
'// DrawDotLine
'// DrawGridLine
'//
'// Clear
'// Refresh
'// Start
'// Connect
'// Point
'//
'////////////////////////////////////////////////



Option Explicit

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
    dst As Any, _
    src As Any, _
    ByVal size As Long)

'control properties
Dim bck_color As OLE_COLOR
Dim grid_color As OLE_COLOR
Dim orgn_color As OLE_COLOR
Dim grph_color As OLE_COLOR

Dim grid_style As DrawStyleConstants
Dim orgn_style As DrawStyleConstants
Dim grph_style As DrawStyleConstants

'drawing parameters
Dim min_x As Double
Dim max_x As Double
Dim min_y As Double
Dim max_y As Double

'distance between grid line
Dim grid_x As Double
Dim grid_y As Double
Dim div_x As Integer
Dim div_y As Integer

'scaling parameters
Dim mx As Double
Dim cx As Double
Dim my As Double
Dim cy As Double

'whether graph drawing has already been started
Dim grph_start As Boolean
Dim curColor As RGBTRIPLE
Dim curX As Long
Dim curY As Long

'for dashed line
Dim curMask As Byte
Dim curMul As Long

'public events
Public Event Resize()
Public Event Paint()
Public Event MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
Public Event MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Public Event MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)

Dim pnt As MPAINTER
Dim autoDraw As Boolean



'////////////////////////////////////////////////
'// interface

Private Sub UserControl_Initialize()

    'default drawing mode
    bck_color = vbBlack
    grid_color = vbCyan
    orgn_color = vbBlack
    grph_color = vbYellow

    grid_style = vbDot
    orgn_style = vbSolid
    grph_style = vbSolid

    'default scaling
    min_x = 0
    max_x = 100
    min_y = 0
    max_y = 100
    
    grid_x = 10
    grid_y = 10
    div_x = 2
    div_y = 2

    autoDraw = False
    CreateMBitmap pnt.dest, 10, 10
    CreatePainter pnt, UserControl.hWnd
    ScalingControl

    autoDraw = True
    grph_start = True
End Sub

Private Sub UserControl_Terminate()
    FreePainter pnt
End Sub

'read and save control properties
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    AutoRedraw = PropBag.ReadProperty("AutoRedraw", AutoRedraw)
    BackColor = PropBag.ReadProperty("BackColor", BackColor)

    GridColor = PropBag.ReadProperty("GridColor", GridColor)
    OriginColor = PropBag.ReadProperty("OriginColor", OriginColor)
    GraphColor = PropBag.ReadProperty("GraphColor", GraphColor)
    
    GridStyle = PropBag.ReadProperty("GridStyle", GridStyle)
    OriginStyle = PropBag.ReadProperty("OriginStyle", OriginStyle)
    GraphStyle = PropBag.ReadProperty("GraphStyle", GraphStyle)
    
    XMin = PropBag.ReadProperty("XMin", XMin)
    XMax = PropBag.ReadProperty("XMax", XMax)
    YMin = PropBag.ReadProperty("YMin", YMin)
    YMax = PropBag.ReadProperty("YMax", YMax)
    
    XGrid = PropBag.ReadProperty("XGrid", XGrid)
    YGrid = PropBag.ReadProperty("YGrid", YGrid)
    XDiv = PropBag.ReadProperty("XDiv", XDiv)
    YDiv = PropBag.ReadProperty("YDiv", YDiv)
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    PropBag.WriteProperty "AutoRedraw", AutoRedraw, True
    PropBag.WriteProperty "BackColor", BackColor, vbBlack

    PropBag.WriteProperty "GridColor", GridColor, RGB(100, 100, 100)
    PropBag.WriteProperty "OriginColor", OriginColor, RGB(128, 128, 128)
    PropBag.WriteProperty "GraphColor", GraphColor, vbYellow
    
    PropBag.WriteProperty "GridStyle", GridStyle, vbDot
    PropBag.WriteProperty "OriginStyle", OriginStyle, vbDashDot
    PropBag.WriteProperty "GraphStyle", GraphStyle, vbSolid
    
    PropBag.WriteProperty "XMin", XMin, 0
    PropBag.WriteProperty "XMax", XMax, 100
    PropBag.WriteProperty "YMin", YMin, 0
    PropBag.WriteProperty "YMax", YMax, 100
    
    PropBag.WriteProperty "XGrid", XGrid, 10
    PropBag.WriteProperty "YGrid", YGrid, 10
    PropBag.WriteProperty "XDiv", XDiv, 2
    PropBag.WriteProperty "YDiv", YDiv, 2
End Sub



'////////////////////////////////////////////////
'// properties

'get bitmap handle
Public Function Handle() As Long
    Handle = pnt.dest.bitPtr
End Function

'get bitmap dimension
Public Sub Dimension(bmpWidth As Long, bmpHeight As Long)
    bmpWidth = pnt.dest.Width
    bmpHeight = pnt.dest.Height
End Sub



'standard autoredraw
Public Property Get AutoRedraw() As Boolean
    AutoRedraw = autoDraw
End Property

Public Property Let AutoRedraw(ByVal vNewValue As Boolean)
    autoDraw = vNewValue
    PropertyChanged "AutoRedraw"
    If vNewValue Then Refresh
End Property

'set or get warna latar secara serta merta
Public Property Get BackColor() As OLE_COLOR
    BackColor = bck_color
End Property

Public Property Let BackColor(ByVal vNewValue As OLE_COLOR)
    bck_color = vNewValue
    UserControl.BackColor = vNewValue
    PropertyChanged "BackColor"
    Clear
End Property



'warna yang akan diguna untuk melukis grid
Public Property Get GridColor() As OLE_COLOR
    GridColor = grid_color
End Property

Public Property Let GridColor(ByVal vNewValue As OLE_COLOR)
    grid_color = vNewValue
    PropertyChanged "GridColor"
End Property

'warna yang akan diguna untuk melukis garis asalan
Public Property Get OriginColor() As OLE_COLOR
    OriginColor = orgn_color
End Property

Public Property Let OriginColor(ByVal vNewValue As OLE_COLOR)
    orgn_color = vNewValue
    PropertyChanged "OriginColor"
End Property

'set or get warna untuk garis graf secara serta merta
Public Property Get GraphColor() As OLE_COLOR
    GraphColor = grph_color
End Property

Public Property Let GraphColor(ByVal vNewValue As OLE_COLOR)
    grph_color = vNewValue
    PropertyChanged "GraphColor"
End Property



'style yang akan diguna untuk melukis grid
Public Property Get GridStyle() As DrawStyleConstants
    GridStyle = grid_style
End Property

Public Property Let GridStyle(ByVal vNewValue As DrawStyleConstants)
    grid_style = vNewValue
    PropertyChanged "GridStyle"
End Property

'style yang akan diguna untuk melukis garis asalan
Public Property Get OriginStyle() As DrawStyleConstants
    OriginStyle = orgn_style
End Property

Public Property Let OriginStyle(ByVal vNewValue As DrawStyleConstants)
    orgn_style = vNewValue
    PropertyChanged "OriginStyle"
End Property

'set or get style untuk garis graf secara serta merta
Public Property Get GraphStyle() As DrawStyleConstants
    GraphStyle = grph_style
End Property

Public Property Let GraphStyle(ByVal vNewValue As DrawStyleConstants)
    grph_style = vNewValue
    PropertyChanged "GraphStyle"
End Property



'////////////////////////////////////////////////
'// limit and grid properties

Public Property Get XMin() As Double
    XMin = min_x
End Property

Public Property Let XMin(ByVal vNewValue As Double)
    If vNewValue >= max_x Then Exit Property
    min_x = vNewValue
    PropertyChanged "XMin"
    ScalingControl
End Property

Public Property Get XMax() As Double
    XMax = max_x
End Property

Public Property Let XMax(ByVal vNewValue As Double)
    If vNewValue <= min_x Then Exit Property
    max_x = vNewValue
    PropertyChanged "XMax"
    ScalingControl
End Property

Public Property Get YMin() As Double
    YMin = min_y
End Property

Public Property Let YMin(ByVal vNewValue As Double)
    If vNewValue >= max_y Then Exit Property
    min_y = vNewValue
    PropertyChanged "YMin"
    ScalingControl
End Property

Public Property Get YMax() As Double
    YMax = max_y
End Property

Public Property Let YMax(ByVal vNewValue As Double)
    If vNewValue <= min_y Then Exit Property
    max_y = vNewValue
    PropertyChanged "YMax"
    ScalingControl
End Property



Public Property Get XGrid() As Double
    XGrid = grid_x
End Property

Public Property Let XGrid(ByVal vNewValue As Double)
    If vNewValue <= 0 Then Exit Property
    grid_x = vNewValue
    PropertyChanged "XGrid"
End Property

Public Property Get YGrid() As Double
    YGrid = grid_y
End Property

Public Property Let YGrid(ByVal vNewValue As Double)
    If vNewValue <= 0 Then Exit Property
    grid_y = vNewValue
    PropertyChanged "YGrid"
End Property

Public Property Get XDiv() As Integer
    XDiv = div_x
End Property

Public Property Let XDiv(ByVal vNewValue As Integer)
    If vNewValue <= 0 Then Exit Property
    div_x = vNewValue
    PropertyChanged "XDiv"
End Property

Public Property Get YDiv() As Integer
    YDiv = div_y
End Property

Public Property Let YDiv(ByVal vNewValue As Integer)
    If vNewValue <= 0 Then Exit Property
    div_y = vNewValue
    PropertyChanged "YDiv"
End Property



'////////////////////////////////////////////////
'// events

Private Sub UserControl_Paint()
    If autoDraw Then
        If Refresh Then RaiseEvent Paint
    End If
End Sub

Private Sub UserControl_Resize()
    ScaleMBitmap pnt.dest, ScaleX(UserControl.Width, ScaleMode, vbPixels), ScaleY(UserControl.Height, ScaleMode, vbPixels)
    pnt.drawn = False
    ScalingControl
    RaiseEvent Resize

    'if not autoredraw, we have to manually clear the bitmap
    If autoDraw Then Clear
End Sub

Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    X = (ScaleX(X, vbTwips, vbPixels) - cx) / mx
    Y = (pnt.dest.Height - ScaleY(Y, vbTwips, vbPixels) - cy) / my
    RaiseEvent MouseDown(Button, Shift, X, Y)
End Sub

Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    X = (ScaleX(X, vbTwips, vbPixels) - cx) / mx
    Y = (pnt.dest.Height - ScaleY(Y, vbTwips, vbPixels) - cy) / my
    RaiseEvent MouseMove(Button, Shift, X, Y)
End Sub

Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    X = (ScaleX(X, vbTwips, vbPixels) - cx) / mx
    Y = (pnt.dest.Height - ScaleY(Y, vbTwips, vbPixels) - cy) / my
    RaiseEvent MouseUp(Button, Shift, X, Y)
End Sub



'////////////////////////////////////////////////
'// limit, grid function

'get or set graph limit
Public Sub Limit(minx As Double, miny As Double, maxx As Double, maxy As Double, Optional retrieve As Boolean = False)

    'setting graph limit
    If Not retrieve Then
        If minx < maxx Then
            min_x = minx
            max_x = maxx
        End If
        If miny < maxy Then
            min_y = miny
            max_y = maxy
        End If
        ScalingControl

    'retrieving graph limit
    Else
        minx = min_x
        maxx = max_x
        miny = min_y
        maxy = max_y
    End If
End Sub

'set scaling/distance between grid line
Public Sub Grid(gridx As Double, gridy As Double, divx As Integer, divy As Integer, Optional retrieve As Boolean = False)

    'setting graph grid scaling
    If Not retrieve Then
        If gridx > 0 Then grid_x = gridx
        If gridy > 0 Then grid_y = gridy
        If divx > 0 Then div_x = divx
        If divy > 0 Then div_y = divy
    
    'retrieving graph grid scaling
    Else
        gridx = grid_x
        gridy = grid_y
        divx = div_x
        divy = div_y
    End If
End Sub



'////////////////////////////////////////////////
'// drawing utilities functions

Public Sub DrawDotLine(x1 As Double, y1 As Double, x2 As Double, y2 As Double, Optional jarakDot As Long = 3, Optional holdDrawing As Boolean = False)
    Dim skipDot As Double
    Dim cp As Double

    'choose line type
    If y1 = y2 Then
        GoSub DrawHorizontalLine
    ElseIf x1 = x2 Then
        GoSub DrawVerticalLine
    Else
        GoSub DrawSlopeLine
    End If

    pnt.drawn = False
    If Not holdDrawing Then Refresh
Exit Sub

'draw horizontal dot line
DrawHorizontalLine:

    'get picture width in pixel and skipDot in scale mode
    skipDot = CDbl(jarakDot * Abs(max_x - min_x)) / pnt.dest.Width
    If x1 > x2 Then skipDot = -skipDot

    'draw horizontal dot line
    For cp = x1 To x2 Step skipDot
        PointBitmap pnt.dest, graphCoorX(cp), graphCoorY(y1), curColor
    Next cp

    'draw last dot
    PointBitmap pnt.dest, graphCoorX(x2), graphCoorY(y1), curColor
Return

'draw vertical dot line
DrawVerticalLine:
    
    'get picture height in pixel and skipDot in scale mode
    skipDot = CDbl(jarakDot * Abs(max_y - min_y)) / pnt.dest.Height
    If y1 > y2 Then skipDot = -skipDot

    'draw vertical dot line
    For cp = y1 To y2 Step skipDot
        PointBitmap pnt.dest, graphCoorX(x1), graphCoorY(cp), curColor
    Next cp
    
    'draw last dot
    PointBitmap pnt.dest, graphCoorX(x1), graphCoorY(y2), curColor
Return

'draw sloped dot line
DrawSlopeLine:
    Dim px As Double
    Dim py As Double
    Dim xRange As Double
    Dim yRange As Double
    Dim length As Double
    Dim sudut As Double
    Dim dx As Double
    Dim dy As Double
    Dim i As Long
    
    'starting dot
    px = x1
    py = y1
    
    'find length of line
    xRange = x2 - x1
    yRange = y2 - y1
    length = Sqr(xRange * xRange + yRange * yRange)

    'find angle of line
    sudut = Atn(yRange / xRange)

    'find skipDot
    dx = pnt.dest.Width
    dy = pnt.dest.Height
    If dx > dy Then skipDot = jarakDot * Abs(max_x - min_x) / dx
    If dy >= dx Then skipDot = jarakDot * Abs(max_y - min_y) / dy
    
    'find dot increment
    dx = Abs(skipDot * Cos(sudut))
    dy = Abs(skipDot * Sin(sudut))
    If xRange < 0 Then dx = -dx
    If yRange < 0 Then dy = -dy
    length = length / skipDot
    
    'drawing sloped dot line
    For i = 1 To length
        PointBitmap pnt.dest, graphCoorX(px), graphCoorY(py), curColor
        px = px + dx
        py = py + dy
    Next i

    'draw last dot
    PointBitmap pnt.dest, graphCoorX(x2), graphCoorY(y2), curColor
Return
End Sub

Public Sub DrawGridLine(Optional jarakDot As Long = 3, Optional drawOrigin As Boolean = True, Optional holdDrawing As Boolean = False)

    'draw grid
    CopyMemory curColor, rgbTobgr(grid_color), 3
    dashToMask grid_style
    GoSub drawGrid

    'draw origin
    If drawOrigin Then
        CopyMemory curColor, rgbTobgr(orgn_color), 3
        dashToMask orgn_style
        GoSub drawOrigin
    End If

    pnt.drawn = False
    If Not holdDrawing Then Refresh
Exit Sub

drawGrid:
    Dim grid_scale As Double
    Dim grid_end As Double
    Dim cp As Double
    Dim i As Long
    
    'find scaling for small vertical grid division
    grid_scale = grid_x / div_x
    If grid_scale <= 0 Then
        grid_scale = Abs(max_x - min_x) / pnt.dest.Width
    End If
    
    'drawing positive vertical division
    grid_end = max_x
    GoSub DrawLoopVerticalGrid
    
    'drawing negative vertical division
    grid_scale = -grid_scale
    grid_end = min_x
    GoSub DrawLoopVerticalGrid
    
    'find scaling for small horizontal grid division
    grid_scale = grid_y / div_y
    If grid_scale <= 0 Then
        grid_scale = Abs(max_y - min_y) / pnt.dest.Height
    End If
    
    'drawing positive horizontal division
    grid_end = max_y
    GoSub DrawLoopHorizontalGrid
    
    'drawing negative horizontal division
    grid_scale = -grid_scale
    grid_end = min_y
    GoSub DrawLoopHorizontalGrid
Return

DrawLoopVerticalGrid:
    i = 0
    For cp = grid_scale To grid_end Step grid_scale
        i = i + 1
        
        'draw small grid
        If i < div_x Then
            DrawDotLine cp, min_y, cp, max_y, jarakDot, True
            
        'draw main grid
        Else
            i = 0
            If curMask = 255 Then
                LineBitmap pnt.dest, graphCoorX(cp), graphCoorY(min_y), graphCoorX(cp), graphCoorY(max_y), curColor
            Else
                DashBitmap pnt.dest, graphCoorX(cp), graphCoorY(min_y), graphCoorX(cp), graphCoorY(max_y), curMask, curMul, curColor
            End If
        End If
    Next cp
Return

DrawLoopHorizontalGrid:
    i = 0
    For cp = grid_scale To grid_end Step grid_scale
        i = i + 1
        
        'draw small grid
        If i < div_y Then
            DrawDotLine min_x, cp, max_x, cp, jarakDot, True

        'draw main grid
        Else
            i = 0
            If curMask = 255 Then
                LineBitmap pnt.dest, graphCoorX(min_x), graphCoorY(cp), graphCoorX(max_x), graphCoorY(cp), curColor
            Else
                DashBitmap pnt.dest, graphCoorX(min_x), graphCoorY(cp), graphCoorX(max_x), graphCoorY(cp), curMask, curMul, curColor
            End If
        End If
    Next cp
Return

drawOrigin:

    'origin line
    If curMask = 255 Then
        LineBitmap pnt.dest, graphCoorX(0), graphCoorY(min_y), graphCoorX(0), graphCoorY(max_y), curColor
        LineBitmap pnt.dest, graphCoorX(min_x), graphCoorY(0), graphCoorX(max_x), graphCoorY(0), curColor
    Else
        DashBitmap pnt.dest, graphCoorX(0), graphCoorY(min_y), graphCoorX(0), graphCoorY(max_y), curMask, curMul, curColor
        DashBitmap pnt.dest, graphCoorX(min_x), graphCoorY(0), graphCoorX(max_x), graphCoorY(0), curMask, curMul, curColor
    End If
Return
End Sub



'////////////////////////////////////////////////
'// graph drawing functions

'clear all drawing
Public Sub Clear(Optional holdDrawing As Boolean = False)
    Dim col As Long
    Dim i As Long

    'clear and update bitmap background
    col = rgbTobgr(bck_color)
    For i = 0 To pnt.dest.Width - 1
        CopyMemory pnt.dest.bit(3 * i), col, 3
    Next i

    col = 3 * pnt.dest.Width + pnt.dest.pad
    For i = 1 To pnt.dest.Height - 1
        CopyMemory pnt.dest.bit(col * i), pnt.dest.bit(0), col
    Next i

    'redrawing to screen
    pnt.drawn = False
    If Not holdDrawing Then Refresh
End Sub

'clearing graph and updating scaling
Public Function Refresh() As Boolean
    Refresh = PaintPainter(pnt, UserControl.hDC)
End Function

Public Function UpdateColor(col As OLE_COLOR)
    CopyMemory curColor, rgbTobgr(col), 3
End Function

'start new graph line
Public Sub start()
    grph_start = True
    UpdateColor grph_color
    dashToMask grph_style
End Sub

'draw and connect graph data and line
Public Sub Connect(X As Double, Y As Double, Optional holdDrawing As Boolean = False)
    Dim ix As Long
    Dim iy As Long

    'plotting next point
    If Not grph_start Then
        ix = graphCoorX(X)
        iy = graphCoorY(Y)
        
        If curMask = 255 Then
            LineBitmap pnt.dest, curX, curY, ix, iy, curColor
        Else
            DashBitmap pnt.dest, curX, curY, ix, iy, curMask, curMul, curColor
        End If
    
        curX = ix
        curY = iy

        pnt.drawn = False
        If Not holdDrawing Then Refresh

    'plotting 1st point
    Else
        grph_start = False
        curX = graphCoorX(X)
        curY = graphCoorY(Y)
    End If
End Sub

Public Sub Point(X As Double, Y As Double, Optional holdDrawing As Boolean = False)
    PointBitmap pnt.dest, graphCoorX(X), graphCoorY(Y), curColor
    pnt.drawn = False
    If Not holdDrawing Then Refresh
End Sub



'////////////////////////////////////////////////
'// scaling and general functions

'ubah graf control scaling
Private Sub ScalingControl()
    
    'scaling graph width
    If min_x <> max_x Then
        mx = CDbl(pnt.dest.Width - 1) / (max_x - min_x)
        cx = -mx * min_x
    End If

    'scaling graph height
    If min_y <> max_y Then
        my = CDbl(pnt.dest.Height - 1) / (max_y - min_y)
        cy = -my * min_y
    End If
End Sub

Private Function graphCoorX(ByRef X As Double) As Long
    graphCoorX = CLng(mx * X + cx)
End Function

Private Function graphCoorY(ByRef Y As Double) As Long
    graphCoorY = CLng(my * Y + cy)
End Function

Private Function rgbTobgr(oleColor As Long) As Long
    rgbTobgr = ((oleColor And 16711680) \ 65536)
    rgbTobgr = rgbTobgr + (oleColor And 65280)
    rgbTobgr = rgbTobgr + (oleColor And 255) * 65536
End Function

'set jenis dash, tukar vb dash kpd soa's bresenham dash
Private Sub dashToMask(style As DrawStyleConstants)
    If style = vbDash Then
        curMask = 15
        curMul = 3
    ElseIf style = vbDashDot Then
        curMask = 95
        curMul = 1
    ElseIf style = vbDashDotDot Then
        curMask = 87
        curMul = 3
    ElseIf style = vbDot Then
        curMask = 85
        curMul = 2
    Else 'solid
        curMask = 255
        curMul = 1
    End If
End Sub

