Attribute VB_Name = "MultimediaPainter"
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' MBitmap painter to Windows
' this file is to be inserted into application
' not to be modified
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' dependency
' - MultimediaBitmap.bas
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' public member
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' ForceXPToCreateCompatibleDC
' ForceXPToCreateCompatibleBitmap
' CreatePainter
' FreePainter
' PaintPainter
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Windows XP Problem:
' CreateCompatibleDC and CreateCompatibleBitmap usually
' failed in Windows XP, to solve this we use force (While)
' in CreatePainter to return successfully, and MAX_XP_TRY
' is the maximum attempt before we really failed to
' create a painter. Shit! to Windows XP!
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Public Const MAX_XP_TRY As Long = 100

Public Const DIB_PAL_COLORS = 1
Public Const SRCCOPY = &HCC0020

Public Declare Function GetDC Lib "user32" ( _
  ByVal hWnd As Long) As Long
Public Declare Function ReleaseDC Lib "user32" ( _
  ByVal hWnd As Long, ByVal hDC As Long) As Long
Public Declare Function DeleteDC Lib "gdi32" ( _
  ByVal hDC As Long) As Long

Public Declare Function SelectObject Lib "gdi32" ( _
  ByVal hDC As Long, _
  ByVal hObject As Long) As Long
Public Declare Function DeleteObject Lib "gdi32" ( _
  ByVal hObject As Long) As Long

Public Declare Function CreateCompatibleDC Lib "gdi32" ( _
  ByVal hDC As Long) As Long
Public Declare Function CreateCompatibleBitmap Lib "gdi32" ( _
  ByVal hDC As Long, _
  ByVal nWidth As Long, _
  ByVal nHeight As Long) As Long

Public Declare Function SetDIBits Lib "gdi32" ( _
  ByVal hDC As Long, _
  ByVal hBitmap As Long, _
  ByVal nStartScan As Long, _
  ByVal nNumScans As Long, _
  lpBits As Any, _
  lpBI As BITMAPINFO, _
  ByVal wUsage As Long) As Long
Public Declare Function BitBlt Lib "gdi32" ( _
  ByVal hDestDC As Long, _
  ByVal X As Long, _
  ByVal Y As Long, _
  ByVal nWidth As Long, _
  ByVal nHeight As Long, _
  ByVal hSrcDC As Long, _
  ByVal xSrc As Long, _
  ByVal ySrc As Long, _
  ByVal dwRop As Long) As Long

Public Type MPAINTER
  dest As MBITMAP
  drawn As Boolean 'set to false when dest is changed
  hwd As Long 'window handle
  wdc As Long 'window dc
  hDC As Long 'memory dc
  hbm As Long 'bitmap handle
  obm As Long 'original hbm for hdc
  Width As Long 'hbm width
  Height As Long 'hbm height
End Type

Public Function ForceXPToCreateCompatibleDC(hDC As Long) As Long
  Dim numXPTry As Long
  Dim res As Long

  'create compatible device context
  Do While res <= 0
    res = CreateCompatibleDC(hDC)
    numXPTry = numXPTry + 1
    If numXPTry >= MAX_XP_TRY Then Exit Do
  Loop

  ForceXPToCreateCompatibleDC = res
End Function

Public Function ForceXPToCreateCompatibleBitmap(hDC As Long, wdt As Long, hgt As Long) As Long
  Dim numXPTry As Long
  Dim res As Long

  'creating compatible bitmap handle
  Do While res <= 0
    res = CreateCompatibleBitmap(hDC, wdt, hgt)
    numXPTry = numXPTry + 1
    If numXPTry >= MAX_XP_TRY Then Exit Do
  Loop

  ForceXPToCreateCompatibleBitmap = res
End Function

Public Function CreatePainter(pnt As MPAINTER, hWnd As Long) As Boolean
  If IsBitmapEmpty(pnt.dest) Then Exit Function
  FreePainter pnt

  pnt.wdc = GetDC(hWnd)
  If pnt.wdc = 0 Then Exit Function
  pnt.hwd = hWnd

  'create compatible device context
  pnt.hDC = ForceXPToCreateCompatibleDC(pnt.wdc)

  'if fail to create dc then reset to none
  If pnt.hDC <= 0 Then
    FreePainter pnt
    Exit Function
  End If

  'creating bitmap handle
  pnt.hbm = ForceXPToCreateCompatibleBitmap(pnt.wdc, pnt.dest.Width, pnt.dest.Height)

  'if fail to create bmp then reset to none
  If pnt.hbm <= 0 Then
    FreePainter pnt
    Exit Function
  End If

  'set painter parameter
  pnt.Width = pnt.dest.Width
  pnt.Height = pnt.dest.Height

  'drawing MBitmap into hdc
  SetDIBits pnt.hDC, pnt.hbm, 0, pnt.Height, pnt.dest.bit(0), pnt.dest.info, DIB_RGB_COLORS
  pnt.obm = SelectObject(pnt.hDC, pnt.hbm)
  pnt.drawn = True

  CreatePainter = True
End Function

Public Function FreePainter(pnt As MPAINTER)

  'freeing bitmap
  If pnt.hbm > 0 Then
    SelectObject pnt.hDC, pnt.obm
    DeleteObject pnt.hbm
  End If
  pnt.hbm = 0
  pnt.obm = 0

  'freeing memory dc
  If pnt.hDC > 0 Then
    DeleteDC pnt.hDC
  End If
  pnt.hDC = 0

  'freeing window dc
  If pnt.wdc > 0 Then
    ReleaseDC pnt.hwd, pnt.wdc
  End If
  pnt.hwd = 0
  pnt.wdc = 0

  'setPainterToNone
  pnt.drawn = False
  pnt.Width = 0
  pnt.Height = 0

  FreePainter = True
End Function

Public Function PaintPainter(pnt As MPAINTER, hDC As Long, Optional xpos As Long = 0, Optional ypos As Long = 0) As Boolean
  Dim resizing As Boolean

  If pnt.hDC <= 0 Then Exit Function

  'recreating or updating bitmap handle
  If Not pnt.drawn Then

    'check dimension changing
    If pnt.Width <> pnt.dest.Width Then resizing = True
    If pnt.Height <> pnt.dest.Height Then resizing = True

    'resizing bitmap
    If resizing Then
      If IsBitmapEmpty(pnt.dest) Then
        FreePainter pnt
        Exit Function
      End If

      SelectObject pnt.hDC, pnt.obm
      DeleteObject pnt.hbm
      pnt.hbm = ForceXPToCreateCompatibleBitmap(pnt.wdc, pnt.dest.Width, pnt.dest.Height)
      If pnt.hbm <= 0 Then
        FreePainter pnt
        Exit Function
      End If

      pnt.Width = pnt.dest.Width
      pnt.Height = pnt.dest.Height
    Else
      SelectObject pnt.hDC, pnt.obm
    End If

    'redrawing bitmap
    SetDIBits pnt.hDC, pnt.hbm, 0, pnt.Height, pnt.dest.bit(0), pnt.dest.info, DIB_RGB_COLORS
    pnt.obm = SelectObject(pnt.hDC, pnt.hbm)
    pnt.drawn = True
  End If

  BitBlt hDC, xpos, ypos, pnt.Width, pnt.Height, pnt.hDC, 0, 0, SRCCOPY

  PaintPainter = True
End Function
